@@ -384,7 +384,8 @@ void mimetype_list_page(void){
   @ <p>The Fossil <a href="%R/help?cmd=/doc">/doc</a> page uses filename
   @ suffixes and the following table to guess at the appropriate mimetype
   @ for each document.</p>
-  @ <table id='mimeTable' border=1 cellpadding=0 class='mimetypetable'>
+  @ <table class='sortable mimetypetable' border=1 cellpadding=0 \
+  @ data-column-types='tt' data-init-sort='1'>
   @ <thead>
   @ <tr><th>Suffix<th>Mimetype
   @ </thead>
@@ -393,7 +394,7 @@ void mimetype_list_page(void){
     @ <tr><td>%h(aMime[i].zSuffix)<td>%h(aMime[i].zMimetype)</tr>
   }
   @ </tbody></table>
-  output_table_sorting_javascript("mimeTable","tt",1);
+  style_table_sorter();
   style_footer();
 }
 
@@ -640,19 +641,27 @@ void doc_page(void){
       }
     }
     if( isUV ){
-      if( db_table_exists("repository","unversioned")
-       && unversioned_content(zName, &filebody)==0
-      ){
-        rid = 1;
-        zDfltTitle = zName;
+      if( db_table_exists("repository","unversioned") ){
+        Stmt q;
+        db_prepare(&q, "SELECT hash, mtime FROM unversioned"
+                       " WHERE name=%Q", zName);
+        if( db_step(&q)==SQLITE_ROW ){
+          etag_check(ETAG_HASH, db_column_text(&q,0));
+          etag_last_modified(db_column_int64(&q,1));
+        }
+        db_finalize(&q);
+        if( unversioned_content(zName, &filebody)==0 ){
+          rid = 1;
+          zDfltTitle = zName;
+        }
       }
     }else if( fossil_strcmp(zCheckin,"ckout")==0 ){
       /* Read from the local checkout */
       char *zFullpath;
       db_must_be_within_tree();
       zFullpath = mprintf("%s/%s", g.zLocalRoot, zName);
-      if( file_isfile(zFullpath)
-       && blob_read_from_file(&filebody, zFullpath)>0 ){
+      if( file_isfile(zFullpath, RepoFILE)
+       && blob_read_from_file(&filebody, zFullpath, RepoFILE)>0 ){
         rid = 1;  /* Fake RID just to get the loop to end */
       }
       fossil_free(zFullpath);
@@ -840,6 +849,7 @@ void logo_page(void){
   Blob logo;
   char *zMime;
 
+  etag_check(ETAG_CONFIG, 0);
   zMime = db_get("logo-mimetype", "image/gif");
   blob_zero(&logo);
   db_blob(&logo, "SELECT value FROM config WHERE name='logo-image'");
@@ -848,7 +858,6 @@ void logo_page(void){
   }
   cgi_set_content_type(zMime);
   cgi_set_content(&logo);
-  g.isConst = 1;
 }
 
 /*
@@ -874,6 +883,7 @@ void background_page(void){
   Blob bgimg;
   char *zMime;
 
+  etag_check(ETAG_CONFIG, 0);
   zMime = db_get("background-mimetype", "image/gif");
   blob_zero(&bgimg);
   db_blob(&bgimg, "SELECT value FROM config WHERE name='background-image'");
@@ -882,7 +892,6 @@ void background_page(void){
   }
   cgi_set_content_type(zMime);
   cgi_set_content(&bgimg);
-  g.isConst = 1;
 }
 
 
