@@ -29,14 +29,17 @@
 ** timeline corresponds to a row in the graph.  GraphRow.idx is 0 for
 ** the top-most row and increases moving down.  Hence (in the absence of
 ** time skew) parents have a larger index than their children.
+**
+** The nParent field is -1 for entires that do not participate in the graph
+** but which are included just so that we can capture their background color.
 */
 struct GraphRow {
   int rid;                    /* The rid for the check-in */
-  i8 nParent;                 /* Number of parents */
+  i8 nParent;                 /* Number of parents.  -1 for technote lines */
   int *aParent;               /* Array of parents.  0 element is primary .*/
   char *zBranch;              /* Branch name */
   char *zBgClr;               /* Background Color */
-  char zUuid[41];             /* Check-in for file ID */
+  char zUuid[HNAME_MAX+1];    /* Check-in for file ID */
 
   GraphRow *pNext;            /* Next row down in the list of all rows */
   GraphRow *pPrev;            /* Previous row */
@@ -181,17 +184,18 @@ int graph_add_row(
   int *aParent,        /* Array of parents */
   const char *zBranch, /* Branch for this check-in */
   const char *zBgClr,  /* Background color. NULL or "" for white. */
-  const char *zUuid,   /* SHA1 hash of the object being graphed */
+  const char *zUuid,   /* hash name of the object being graphed */
   int isLeaf           /* True if this row is a leaf */
 ){
   GraphRow *pRow;
   int nByte;
+  static int nRow = 0;
 
   if( p->nErr ) return 0;
   nByte = sizeof(GraphRow);
-  nByte += sizeof(pRow->aParent[0])*nParent;
+  if( nParent>0 ) nByte += sizeof(pRow->aParent[0])*nParent;
   pRow = (GraphRow*)safeMalloc( nByte );
-  pRow->aParent = (int*)&pRow[1];
+  pRow->aParent = nParent>0 ? (int*)&pRow[1] : 0;
   pRow->rid = rid;
   pRow->nParent = nParent;
   pRow->zBranch = persistBranchName(p, zBranch);
@@ -201,7 +205,7 @@ int graph_add_row(
   memset(pRow->aiRiser, -1, sizeof(pRow->aiRiser));
   if( zBgClr==0 ) zBgClr = "";
   pRow->zBgClr = persistBranchName(p, zBgClr);
-  memcpy(pRow->aParent, aParent, sizeof(aParent[0])*nParent);
+  if( nParent>0 ) memcpy(pRow->aParent, aParent, sizeof(aParent[0])*nParent);
   if( p->pFirst==0 ){
     p->pFirst = pRow;
   }else{
@@ -209,7 +213,7 @@ int graph_add_row(
   }
   p->pLast = pRow;
   p->nRow++;
-  pRow->idx = pRow->idxTop = p->nRow;
+  pRow->idx = pRow->idxTop = ++nRow;
   return pRow->idx;
 }
 
@@ -260,7 +264,6 @@ static void assignChildrenToRail(GraphRow *pBottom){
   GraphRow *pPrior;
   u64 mask = ((u64)1)<<iRail;
 
-  pBottom->iRail = iRail;
   pBottom->railInUse |= mask;
   pPrior = pBottom;
   for(pCurrent=pBottom->pChild; pCurrent; pCurrent=pCurrent->pChild){
@@ -343,14 +346,25 @@ static void riser_to_top(GraphRow *pRow){
 
 /*
 ** Compute the complete graph
+**
+** When primary or merge parents are off-screen, normally a line is drawn
+** from the node down to the bottom of the graph.  This line is called a
+** "descender".  But if the omitDescenders flag is true, then lines down
+** to the bottom of the screen are omitted.
 */
 void graph_finish(GraphContext *p, int omitDescenders){
   GraphRow *pRow, *pDesc, *pDup, *pLoop, *pParent;
-  int i;
+  int i, j;
   u64 mask;
   int hasDup = 0;      /* True if one or more isDup entries */
   const char *zTrunk;
 
+  /* If mergeRiserFrom[X]==Y that means rail X holds a merge riser
+  ** coming up from the bottom of the graph from off-screen check-in Y
+  ** where Y is the RID.  There is no riser on rail X if mergeRiserFrom[X]==0.
+  */
+  int mergeRiserFrom[GR_MAX_RAIL];
+
   if( p==0 || p->pFirst==0 || p->nErr ) return;
   p->nErr = 1;   /* Assume an error until proven otherwise */
 
@@ -368,6 +382,7 @@ void graph_finish(GraphContext *p, int omitDescenders){
     hashInsert(p, pRow, 1);
   }
   p->mxRail = -1;
+  memset(mergeRiserFrom, 0, sizeof(mergeRiserFrom));
 
   /* Purge merge-parents that are out-of-graph if descenders are not
   ** drawn.
@@ -422,7 +437,7 @@ void graph_finish(GraphContext *p, int omitDescenders){
   */
   for(pRow=p->pFirst; pRow; pRow=pRow->pNext){
     if( pRow->isDup ) continue;
-    if( pRow->nParent==0 ) continue;                   /* Root node */
+    if( pRow->nParent<=0 ) continue;                   /* Root node */
     pParent = hashFind(p, pRow->aParent[0]);
     if( pParent==0 ) continue;                         /* Parent off-screen */
     if( pParent->zBranch!=pRow->zBranch ) continue;    /* Different branch */
@@ -445,6 +460,7 @@ void graph_finish(GraphContext *p, int omitDescenders){
   for(i=0; i<2; i++){
     for(pRow=p->pLast; pRow; pRow=pRow->pPrev){
       if( pRow->isDup ) continue;
+      if( pRow->nParent<0 ) continue;
       if( i==0 ){
         if( pRow->zBranch!=zTrunk ) continue;
       }else {
@@ -476,15 +492,13 @@ void graph_finish(GraphContext *p, int omitDescenders){
 
     if( pRow->iRail>=0 ){
       if( pRow->pChild==0 && !pRow->timeWarp ){
-        if( omitDescenders || pRow->isLeaf ){
-          /* no-op */
-        }else{
+        if( !omitDescenders && count_nonbranch_children(pRow->rid)!=0 ){
           riser_to_top(pRow);
         }
       }
       continue;
     }
-    if( pRow->isDup ){
+    if( pRow->isDup || pRow->nParent<0 ){
       continue;
     }else{
       assert( pRow->nParent>0 );
@@ -521,7 +535,7 @@ void graph_finish(GraphContext *p, int omitDescenders){
     pRow->railInUse |= mask;
     if( pRow->pChild ){
       assignChildrenToRail(pRow);
-    }else if( !pRow->isLeaf && !omitDescenders ){
+    }else if( !omitDescenders && count_nonbranch_children(pRow->rid)!=0 ){
       riser_to_top(pRow);
     }
     if( pParent ){
@@ -540,8 +554,18 @@ void graph_finish(GraphContext *p, int omitDescenders){
       pDesc = hashFind(p, parentRid);
       if( pDesc==0 ){
         /* Merge from a node that is off-screen */
-        int iMrail = findFreeRail(p, pRow->idx, p->nRow, 0);
-        if( p->mxRail>=GR_MAX_RAIL ) return;
+        int iMrail = -1;
+        for(j=0; j<GR_MAX_RAIL; j++){
+          if( mergeRiserFrom[j]==parentRid ){
+            iMrail = j;
+            break;
+          }
+        }
+        if( iMrail==-1 ){
+          iMrail = findFreeRail(p, pRow->idx, p->nRow, 0);
+          if( p->mxRail>=GR_MAX_RAIL ) return;
+          mergeRiserFrom[iMrail] = parentRid;
+        }
         mask = BIT(iMrail);
         pRow->mergeIn[iMrail] = 1;
         pRow->mergeDown |= mask;
