@@ -71,22 +71,30 @@ static struct {
   int nFileAlloc;             /* Number of slots in aFile[] */
   ImportFile *aFile;          /* Information about files in a commit */
   int fromLoaded;             /* True zFrom content loaded into aFile[] */
-  int hasLinks;               /* True if git repository contains symlinks */
   int tagCommit;              /* True if the commit adds a tag */
 } gg;
 
 /*
 ** Duplicate a string.
 */
-char *fossil_strdup(const char *zOrig){
+char *fossil_strndup(const char *zOrig, int len){
   char *z = 0;
   if( zOrig ){
-    int n = strlen(zOrig);
+    int n;
+    if( len<0 ){
+      n = strlen(zOrig);
+    }else{
+      for( n=0; zOrig[n] && n<len; ++n );
+    }
     z = fossil_malloc( n+1 );
-    memcpy(z, zOrig, n+1);
+    memcpy(z, zOrig, n);
+    z[n] = 0;
   }
   return z;
 }
+char *fossil_strdup(const char *zOrig){
+  return fossil_strndup(zOrig, -1);
+}
 
 /*
 ** A no-op "xFinish" method
@@ -143,14 +151,14 @@ static void import_reset(int freeAll){
 static int fast_insert_content(
   Blob *pContent,          /* Content to insert */
   const char *zMark,       /* Label using this mark, if not NULL */
-  int saveUuid,            /* Save SHA1 hash in gg.zPrevCheckin */
+  int saveUuid,            /* Save artifact hash in gg.zPrevCheckin */
   int doParse              /* Invoke manifest_crosslink() */
 ){
   Blob hash;
   Blob cmpr;
   int rid;
 
-  sha1sum_blob(pContent, &hash);
+  hname_hash(pContent, 0, &hash);
   rid = db_int(0, "SELECT rid FROM blob WHERE uuid=%B", &hash);
   if( rid==0 ){
     static Stmt ins;
@@ -210,9 +218,12 @@ static void finish_tag(void){
   if( gg.zDate && gg.zTag && gg.zFrom && gg.zUser ){
     blob_zero(&record);
     blob_appendf(&record, "D %s\n", gg.zDate);
-    blob_appendf(&record, "T +%F%F%F %s\n", gimport.zTagPre, gg.zTag,
+    blob_appendf(&record, "T +sym-%F%F%F %s", gimport.zTagPre, gg.zTag,
         gimport.zTagSuf, gg.zFrom);
-    blob_appendf(&record, "U %F\n", gg.zUser);
+    if( gg.zComment ){
+      blob_appendf(&record, " %F", gg.zComment);
+    }
+    blob_appendf(&record, "\nU %F\n", gg.zUser);
     md5sum_blob(&record, &cksum);
     blob_appendf(&record, "Z %b\n", &cksum);
     fast_insert_content(&record, 0, 0, 1);
@@ -267,7 +278,6 @@ static void finish_commit(void){
       blob_append(&record, " x\n", 3);
     }else if( gg.aFile[i].isLink ){
       blob_append(&record, " l\n", 3);
-      gg.hasLinks = 1;
     }else{
       blob_append(&record, "\n", 1);
     }
@@ -507,6 +517,11 @@ static void dequote_git_filename(char *zName){
 }
 
 
+static struct{
+  const char *zMasterName;    /* Name of master branch */
+  int authorFlag;             /* Use author as checkin committer */
+} ggit;
+
 /*
 ** Read the git-fast-import format from pIn and insert the corresponding
 ** content into the database.
@@ -530,10 +545,11 @@ static void git_fast_import(FILE *pIn){
       gg.xFinish = finish_blob;
     }else
     if( strncmp(zLine, "commit ", 7)==0 ){
+      const char *zRefName;
       gg.xFinish();
       gg.xFinish = finish_commit;
       trim_newline(&zLine[7]);
-      z = &zLine[7];
+      zRefName = &zLine[7];
 
       /* The argument to the "commit" line might match either of these
       ** patterns:
@@ -553,11 +569,11 @@ static void git_fast_import(FILE *pIn){
       ** None of the above is explained in the git-fast-export
       ** documentation.  We had to figure it out via trial and error.
       */
-      for(i=5; i<strlen(z) && z[i]!='/'; i++){}
-      gg.tagCommit = strncmp(&z[5], "tags", 4)==0;  /* True for pattern B */
-      if( z[i+1]!=0 ) z += i+1;
-      if( fossil_strcmp(z, "master")==0 ) z = "trunk";
-      gg.zBranch = fossil_strdup(z);
+      for(i=5; i<strlen(zRefName) && zRefName[i]!='/'; i++){}
+      gg.tagCommit = strncmp(&zRefName[5], "tags", 4)==0;  /* True for pattern B */
+      if( zRefName[i+1]!=0 ) zRefName += i+1;
+      if( fossil_strcmp(zRefName, "master")==0 ) zRefName = ggit.zMasterName;
+      gg.zBranch = fossil_strdup(zRefName);
       gg.fromLoaded = 0;
     }else
     if( strncmp(zLine, "tag ", 4)==0 ){
@@ -594,15 +610,21 @@ static void git_fast_import(FILE *pIn){
         if( got!=gg.nData ){
           fossil_fatal("short read: got %d of %d bytes", got, gg.nData);
         }
-        gg.aData[got] = 0;
-        if( gg.zComment==0 && gg.xFinish==finish_commit ){
+        gg.aData[got] = '\0';
+        if( gg.zComment==0 &&
+            (gg.xFinish==finish_commit || gg.xFinish==finish_tag) ){
+	  /* Strip trailing newline, it's appended to the comment. */
+	  if( gg.aData[got-1] == '\n' )
+	    gg.aData[got-1] = '\0';
           gg.zComment = gg.aData;
           gg.aData = 0;
           gg.nData = 0;
         }
       }
     }else
-    if( strncmp(zLine, "author ", 7)==0 ){
+    if( (!ggit.authorFlag && strncmp(zLine, "author ", 7)==0)
+        || (ggit.authorFlag && strncmp(zLine, "committer ",10)==0
+            && gg.zUser!=NULL) ){
       /* No-op */
     }else
     if( strncmp(zLine, "mark ", 5)==0 ){
@@ -610,19 +632,28 @@ static void git_fast_import(FILE *pIn){
       fossil_free(gg.zMark);
       gg.zMark = fossil_strdup(&zLine[5]);
     }else
-    if( strncmp(zLine, "tagger ", 7)==0 || strncmp(zLine, "committer ",10)==0 ){
+    if( strncmp(zLine, "tagger ", 7)==0
+        || (ggit.authorFlag && strncmp(zLine, "author ", 7)==0)
+        || strncmp(zLine, "committer ",10)==0 ){
       sqlite3_int64 secSince1970;
-      for(i=0; zLine[i] && zLine[i]!='<'; i++){}
-      if( zLine[i]==0 ) goto malformed_line;
-      z = &zLine[i+1];
-      for(i=i+1; zLine[i] && zLine[i]!='>'; i++){}
-      if( zLine[i]==0 ) goto malformed_line;
-      zLine[i] = 0;
+      z = strchr(zLine, ' ');
+      while( fossil_isspace(*z) ) z++;
+      if( (zTo=strchr(z, '>'))==NULL ) goto malformed_line;
+      *(++zTo) = '\0';
+      /* Lookup user by contact info. */
       fossil_free(gg.zUser);
-      gg.zUser = fossil_strdup(z);
+      gg.zUser = db_text(0, "SELECT login FROM user WHERE info=%Q", z);
+      if( gg.zUser==NULL ){
+        /* If there is no user with this contact info,
+	 * then use the email address as the username. */
+        if ( (z=strchr(z, '<'))==NULL ) goto malformed_line;
+        z++;
+        *(zTo-1) = '\0';
+        gg.zUser = fossil_strdup(z);
+      }
       secSince1970 = 0;
-      for(i=i+2; fossil_isdigit(zLine[i]); i++){
-        secSince1970 = secSince1970*10 + zLine[i] - '0';
+      for(zTo++; fossil_isdigit(*zTo); zTo++){
+        secSince1970 = secSince1970*10 + *zTo - '0';
       }
       fossil_free(gg.zDate);
       gg.zDate = db_text(0, "SELECT datetime(%lld, 'unixepoch')", secSince1970);
@@ -742,9 +773,6 @@ static void git_fast_import(FILE *pIn){
     }
   }
   gg.xFinish();
-  if( gg.hasLinks ){
-    db_set_int("allow-symlinks", 1, 0);
-  }
   import_reset(1);
   return;
 
@@ -1247,6 +1275,30 @@ static int svn_parse_path(char *zPath, char **zFile, int *type){
   return branchId;
 }
 
+/*
+** Insert content of corresponding content blob into the database.
+** If content is identified as a symbolic link, then trailing
+** "link " characters are removed from content.
+**
+** content is considered to be a symlink if zPerm contains at least
+** one "l" character.
+*/
+static int svn_handle_symlinks(const char *perms, Blob *content){
+  Blob link_blob;
+  if( perms && strstr(perms, "l")!=0 ){
+    if( blob_size(content)>5 ){
+      /* Skip trailing 'link ' characters */
+      blob_seek(content, 5, BLOB_SEEK_SET);
+      blob_tail(content, &link_blob);
+      return content_put(&link_blob);
+    }else{
+      fossil_fatal("Too short symbolic link path");
+    }
+  }else{
+    return content_put(content);
+  }
+}
+
 /*
 ** Read the svn-dump format from pIn and insert the corresponding
 ** content into the database.
@@ -1347,6 +1399,10 @@ static void svn_dump_import(FILE *pIn){
       char *zPerm = svn_find_prop(rec, "svn:executable") ? "x" : 0;
       int deltaFlag = 0;
       int srcRev = 0;
+
+      if ( zPerm==0 ){
+        zPerm = svn_find_prop(rec, "svn:special") ? "l" : 0;
+      }
       if( branchId==0 ){
         svn_free_rec(&rec);
         continue;
@@ -1458,9 +1514,15 @@ static void svn_dump_import(FILE *pIn){
               blob_zero(&deltaSrc);
             }
             svn_apply_svndiff(&rec.content, &deltaSrc, &target);
-            rid = content_put(&target);
+            rid = svn_handle_symlinks(zPerm, &target);
           }else if( rec.contentFlag ){
-            rid = content_put(&rec.content);
+            rid = svn_handle_symlinks(zPerm, &rec.content);
+          }else if( zSrcPath ){
+            if ( zPerm==0 ){
+              zPerm = db_text(0, "SELECT tperm FROM xfiles"
+                                 " WHERE tpath=%Q AND tbranch=%d"
+                                 "", zSrcPath, branchId);
+            }
           }
           db_bind_text(&addFile, ":path", zFile);
           db_bind_int(&addFile, ":branch", branchId);
@@ -1479,6 +1541,12 @@ static void svn_dump_import(FILE *pIn){
           fossil_fatal("Missing Node-kind");
         }
         if( rec.contentFlag && strncmp(zKind, "dir", 3)!=0 ){
+          if ( zPerm==0 ){
+            zPerm = db_text(0, "SELECT tperm FROM xfiles"
+                               " WHERE tpath=%Q AND tbranch=%d"
+                               "", zFile, branchId);
+          }
+
           if( deltaFlag ){
             Blob deltaSrc;
             Blob target;
@@ -1488,9 +1556,9 @@ static void svn_dump_import(FILE *pIn){
                             ")", zFile, branchId);
             content_get(rid, &deltaSrc);
             svn_apply_svndiff(&rec.content, &deltaSrc, &target);
-            rid = content_put(&target);
+            rid = svn_handle_symlinks(zPerm, &target);
           }else{
-            rid = content_put(&rec.content);
+            rid = svn_handle_symlinks(zPerm, &rec.content);
           }
           db_bind_text(&addFile, ":path", zFile);
           db_bind_int(&addFile, ":branch", branchId);
@@ -1539,8 +1607,10 @@ static void svn_dump_import(FILE *pIn){
 **
 **   --git        Import from the git-fast-export file format (default)
 **                Options:
-**                  --import-marks FILE Restore marks table from FILE
-**                  --export-marks FILE Save marks table to FILE
+**                  --import-marks  FILE Restore marks table from FILE
+**                  --export-marks  FILE Save marks table to FILE
+**                  --rename-master NAME Renames the master branch to NAME
+**                  --use-author    Uses author as the committer
 **
 **   --svn        Import from the svnadmin-dump file format.  The default
 **                behaviour (unless overridden by --flat) is to treat 3
@@ -1614,7 +1684,7 @@ void import_cmd(void){
     {"rename-rev"   , &gsvn.zRevPre, "svn-rev-", &gsvn.zRevSuf      , "", 2},
   }, *renOpt = renOpts;
   int i;
-  for( i = 0; i < sizeof(renOpts) / sizeof(*renOpts); ++i, ++renOpt ){
+  for( i = 0; i < count(renOpts); ++i, ++renOpt ){
     if( 1 << svnFlag & renOpt->format ){
       const char *zArgument = find_option(renOpt->zOpt, 0, 1);
       if( zArgument ){
@@ -1646,7 +1716,7 @@ void import_cmd(void){
     unsigned nIgnTree = 0;
     while( (zIgnTree = find_option("ignore-tree", 0, 1)) ){
       if ( *zIgnTree ){
-        gsvn.azIgnTree = fossil_realloc(gsvn.azIgnTree,
+        gsvn.azIgnTree = fossil_realloc((void *)gsvn.azIgnTree,
             sizeof(*gsvn.azIgnTree) * (nIgnTree + 2));
         gsvn.azIgnTree[nIgnTree++] = zIgnTree;
         gsvn.azIgnTree[nIgnTree] = 0;
@@ -1662,6 +1732,10 @@ void import_cmd(void){
   }else if( gitFlag ){
     markfile_in = find_option("import-marks", 0, 1);
     markfile_out = find_option("export-marks", 0, 1);
+    if( !(ggit.zMasterName = find_option("rename-master", 0, 1)) ){
+      ggit.zMasterName = "master";
+    }
+    ggit.authorFlag = find_option("use-author", 0, 0)!=0;
   }
   verify_all_options();
 
@@ -1670,6 +1744,7 @@ void import_cmd(void){
   }
   if( g.argc==4 ){
     pIn = fossil_fopen(g.argv[3], "rb");
+    if( pIn==0 ) fossil_fatal("cannot open input file \"%s\"", g.argv[3]);
   }else{
     pIn = stdin;
     fossil_binary_mode(pIn);
@@ -1682,7 +1757,10 @@ void import_cmd(void){
   db_open_config(0, 0);
 
   db_begin_transaction();
-  if( !incrFlag ) db_initial_setup(0, 0, 0);
+  if( !incrFlag ){
+    db_initial_setup(0, 0, 0);
+    db_set("main-branch", gimport.zTrunkName, 0);
+  }
 
   if( svnFlag ){
     db_multi_exec(
@@ -1763,6 +1841,7 @@ void import_cmd(void){
     */
     db_multi_exec(
        "CREATE TEMP TABLE xmark(tname TEXT UNIQUE, trid INT, tuuid TEXT);"
+       "CREATE INDEX temp.i_xmark ON xmark(trid);"
        "CREATE TEMP TABLE xbranch(tname TEXT UNIQUE, brnm TEXT);"
        "CREATE TEMP TABLE xtag(tname TEXT UNIQUE, tcontent TEXT);"
     );
@@ -1770,10 +1849,10 @@ void import_cmd(void){
     if( markfile_in ){
       FILE *f = fossil_fopen(markfile_in, "r");
       if( !f ){
-        fossil_fatal("cannot open %s for reading\n", markfile_in);
+        fossil_fatal("cannot open %s for reading", markfile_in);
       }
-      if(import_marks(f, &blobs, NULL)<0){
-        fossil_fatal("error importing marks from file: %s\n", markfile_in);
+      if( import_marks(f, &blobs, NULL, NULL)<0 ){
+        fossil_fatal("error importing marks from file: %s", markfile_in);
       }
       fclose(f);
     }
@@ -1797,9 +1876,9 @@ void import_cmd(void){
         rid = db_column_int(&q_marks, 0);
         if( db_int(0, "SELECT count(objid) FROM event"
                       " WHERE objid=%d AND type='ci'", rid)==0 ){
-          if( bag_find(&blobs, rid)==0 ){
-            bag_insert(&blobs, rid);
-          }
+          /* Blob marks exported by git aren't saved between runs, so they need
+          ** to be left free for git to re-use in the future.
+          */
         }else{
           bag_insert(&vers, rid);
         }
@@ -1807,7 +1886,7 @@ void import_cmd(void){
       db_finalize(&q_marks);
       f = fossil_fopen(markfile_out, "w");
       if( !f ){
-        fossil_fatal("cannot open %s for writing\n", markfile_out);
+        fossil_fatal("cannot open %s for writing", markfile_out);
       }
       export_marks(f, &blobs, &vers);
       fclose(f);
