@@ -361,6 +361,7 @@ void tag_add_artifact(
     manifest_crosslink(nrid, &ctrl, MC_PERMIT_HOOKS);
   }
   assert( blob_is_reset(&ctrl) );
+  manifest_to_disk(rid);
 }
 
 /*
@@ -382,9 +383,9 @@ void tag_add_artifact(
 **           --propagate                 Propagating tag.
 **           --date-override DATETIME    Set date and time added.
 **           --user-override USER        Name USER when adding the tag.
-**           --dryrun|-n                 Display the tag text, but to not
+**           --dryrun|-n                 Display the tag text, but do not
 **                                       actually insert it into the database.
-**         
+**
 **         The --date-override and --user-override options support
 **         importing history from other SCM systems. DATETIME has
 **         the form 'YYYY-MMM-DD HH:MM:SS'.
@@ -400,7 +401,7 @@ void tag_add_artifact(
 **         List all objects that use TAGNAME.  TYPE can be "ci" for
 **         check-ins or "e" for events. The limit option limits the number
 **         of results to the given value.
-**         
+**
 **         Options:
 **           --raw           Raw tag name.
 **           -t|--type TYPE  One of "ci", or "e".
@@ -516,7 +517,7 @@ void tag_cmd(void){
                     " SELECT rid FROM tagxref"
                     "  WHERE tagtype>0 AND tagid=%d"
                     ")"
-          " ORDER BY event.mtime DESC",
+          " ORDER BY event.mtime DESC /*sort*/",
           timeline_query_for_tty(), zType, tagid
         );
         db_prepare(&q, "%s", blob_sql_text(&sql));
@@ -588,7 +589,7 @@ tag_cmd_usage:
 /*
 ** COMMAND: reparent*
 **
-** Usage: %fossil reparent [OPTIONS] CHECK-IN PARENT ....
+** Usage: %fossil reparent [OPTIONS] CHECK-IN PARENT ...
 **
 ** Create a "parent" tag that causes CHECK-IN to be interpreted as a
 ** child of PARENT.  If multiple PARENTs are listed, then the first is
@@ -620,7 +621,7 @@ void reparent_cmd(void){
   db_find_and_open_repository(0, 0);
   verify_all_options();
   if( g.argc<4 ){
-    usage("reparent [OPTIONS] PARENT ...");
+    usage("[OPTIONS] CHECK-IN PARENT ...");
   }
   rid = name_to_typed_rid(g.argv[2], "ci");
   blob_init(&value, 0, 0);
@@ -628,7 +629,7 @@ void reparent_cmd(void){
     int pid = name_to_typed_rid(g.argv[i], "ci");
     if( i>3 ) blob_append(&value, " ", 1);
     zUuid = rid_to_uuid(pid);
-    blob_append(&value, zUuid, UUID_SIZE);
+    blob_append(&value, zUuid, strlen(zUuid));
     fossil_free(zUuid);
   }
   if( bTest && !dryRun ){
@@ -655,7 +656,7 @@ void taglist_page(void){
   login_anonymous_available();
   style_header("Tags");
   style_adunit_config(ADUNIT_RIGHT_OK);
-  style_submenu_element("Timeline", "Timeline", "tagtimeline");
+  style_submenu_element("Timeline", "tagtimeline");
   @ <h2>Non-propagating tags:</h2>
   db_prepare(&q,
     "SELECT substr(tagname,5)"
@@ -670,7 +671,7 @@ void taglist_page(void){
   while( db_step(&q)==SQLITE_ROW ){
     const char *zName = db_column_text(&q, 0);
     if( g.perm.Hyperlink ){
-      @ <li>%z(xhref("class='taglink'","%R/timeline?t=%T&n=200",zName))
+      @ <li>%z(chref("taglink","%R/timeline?t=%T&n=200",zName))
       @ %h(zName)</a></li>
     }else{
       @ <li><span class="tagDsp">%h(zName)</span></li>
@@ -694,7 +695,7 @@ void tagtimeline_page(void){
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
 
   style_header("Tagged Check-ins");
-  style_submenu_element("List", "List", "taglist");
+  style_submenu_element("List", "taglist");
   login_anonymous_available();
   @ <h2>Check-ins with non-propagating tags:</h2>
   db_prepare(&q,
@@ -702,7 +703,7 @@ void tagtimeline_page(void){
     "                     WHERE tagtype=1 AND srcid>0"
     "                       AND tagid IN (SELECT tagid FROM tag "
     "                                      WHERE tagname GLOB 'sym-*'))"
-    " ORDER BY event.mtime DESC",
+    " ORDER BY event.mtime DESC /*sort*/",
     timeline_query_for_www()
   );
   www_print_timeline(&q, 0, 0, 0, 0, 0);
