@@ -426,20 +426,26 @@ static void tar_finish(Blob *pOut){
 **
 ** Generate a GZIP-compressed tarball in the file given by the first argument
 ** that contains files given in the second and subsequent arguments.
+**
+**   -h, --dereference   Follow symlinks; archive the files they point to.
 */
 void test_tarball_cmd(void){
   int i;
   Blob zip;
+  int eFType = SymFILE;
   if( g.argc<3 ){
-    usage("ARCHIVE FILE....");
+    usage("ARCHIVE [options] FILE....");
+  }
+  if( find_option("dereference","h",0) ){
+    eFType = ExtFILE;
   }
   sqlite3_open(":memory:", &g.db);
   tar_begin(-1);
   for(i=3; i<g.argc; i++){
     Blob file;
     blob_zero(&file);
-    blob_read_from_file(&file, g.argv[i]);
-    tar_add_file(g.argv[i], &file, file_wd_perm(0), file_wd_mtime(0));
+    blob_read_from_file(&file, g.argv[i], eFType);
+    tar_add_file(g.argv[i], &file, file_perm(0,0), file_mtime(0,0));
     blob_reset(&file);
   }
   tar_finish(&zip);
@@ -643,6 +649,35 @@ void tarball_cmd(void){
   blob_reset(&tarball);
 }
 
+/*
+** Check to see if the input string is of the form:
+**
+**        checkin-name/filename.ext
+**
+** In other words, check to see if the input contains a single '/'
+** character that separates a valid check-in name from a filename.
+**
+** If the condition is true, return the check-in name and set the
+** input string to be the filename.
+**
+** If the condition is false, return NULL
+*/
+char *tar_uuid_from_name(char **pzName){
+  char *zName = *pzName;
+  int i, n;
+  for(i=n=0; zName[i]; i++){
+    if( zName[i]=='/' ){
+      if( n==0 ) n = i;
+      else return 0;
+    }
+  }
+  if( n==0 ) return 0;
+  if( zName[n+1]==0 ) return 0;
+  zName[n] = 0;
+  *pzName = fossil_strdup(&zName[n+1]);
+  return zName;
+}
+
 /*
 ** WEBPAGE: tarball
 ** URL: /tarball
@@ -651,6 +686,14 @@ void tarball_cmd(void){
 ** query parameter.  Return that compressed tarball as the HTTP reply
 ** content.
 **
+** The r= and name= query parameters can be specified as extensions to the
+** URI.  Example, the following URIs are all equivalent:
+**
+**      /tarball/release/xyz.tar.gz
+**      /tarball?r=release&name=xyz.tar.gz
+**      /tarball/xyz.tar.gz?r=release
+**      /tarball?name=release/xyz.tar.gz
+**
 ** Query parameters:
 **
 **   name=NAME[.tar.gz]  The base name of the output file.  The default
@@ -661,8 +704,11 @@ void tarball_cmd(void){
 **   r=TAG               The check-in that is turned into a compressed tarball.
 **                       Defaults to "trunk".  This query parameter used to
 **                       be called "uuid" and "uuid" is still accepted for
-**                       backwards compatibility.  If omitted, the default
-**                       check-in name is "trunk".
+**                       backwards compatibility.  If the name= query parameter
+**                       contains one "/" character then the part before the /
+**                       is the TAG and the part after the / is the true name.
+**                       If no TAG is specified by any of the above means, then
+**                       "trunk" is used as the default.
 **
 **   in=PATTERN          Only include files that match the comma-separate
 **                       list of GLOB patterns in PATTERN, as with ex=
@@ -687,10 +733,10 @@ void tarball_page(void){
   login_check_credentials();
   if( !g.perm.Zip ){ login_needed(g.anon.Zip); return; }
   load_control();
-  zName = mprintf("%s", PD("name",""));
-  nName = strlen(zName);
+  zName = fossil_strdup(PD("name",""));
   z = P("r");
   if( z==0 ) z = P("uuid");
+  if( z==0 ) z = tar_uuid_from_name(&zName);
   if( z==0 ) z = "trunk";
   g.zOpenRevision = zRid = fossil_strdup(z);
   nRid = strlen(zRid);
@@ -698,6 +744,7 @@ void tarball_page(void){
   if( zInclude ) pInclude = glob_create(zInclude);
   zExclude = P("ex");
   if( zExclude ) pExclude = glob_create(zExclude);
+  nName = strlen(zName);
   if( nName>7 && fossil_strcmp(&zName[nName-7], ".tar.gz")==0 ){
     /* Special case:  Remove the ".tar.gz" suffix.  */
     nName -= 7;
@@ -727,6 +774,7 @@ void tarball_page(void){
   if( zInclude ) blob_appendf(&cacheKey, ",in=%Q", zInclude);
   if( zExclude ) blob_appendf(&cacheKey, ",ex=%Q", zExclude);
   zKey = blob_str(&cacheKey);
+  etag_check(ETAG_HASH, zKey);
 
   if( P("debug")!=0 ){
     style_header("Tarball Generator Debug Screen");
